import logging
from OpenHub.calibrators.raw_value_converter import RawValueConverter
from OpenHub import utilities
import time


class SoilMoistureVoltageConverter(RawValueConverter):
    logger = logging.getLogger(__name__)
    calibration_directory = None
    calibration_y_vals = [99.9, 0.01]

    def __init__(self, accessory_serial_no=None, m=None, b=None, max_voltage=0.1, min_voltage=100.0):
        self.m = m
        self.b = b
        self.max_voltage = max_voltage
        self.min_voltage = min_voltage
        self.homekit_accessory_serial_no = accessory_serial_no
        super().__init__()

    def calibrate(self, channel):
        input("Please Dry Sensor For Channel " + str(channel.index))
        for _ in range(5):
            voltage = channel.get_raw_value()
            self.set_max_voltage(voltage)
            time.sleep(1)

        input("Please Place Sensor For Channel " + str(channel.index) + " in Water")
        for _ in range(5):
            voltage = channel.get_raw_value()
            self.set_min_voltage(voltage)
            time.sleep(1)

        self.ensure_min_max_different()
        self.set_slope_and_intercept_for_calibration()

    def ensure_min_max_different(self):
        if self.max_voltage == 0:
            self.max_voltage = 3
        if self.min_voltage == 0:
            self.min_voltage = .5
        if self.max_voltage == self.min_voltage:
            self.max_voltage = self.min_voltage + .1

    def set_slope_and_intercept_for_calibration(self):
        self.logger.info("Calculating new slope and intercept for output computation")
        self.m, self.b = utilities.best_fit([(1 / self.min_voltage), (1 / self.max_voltage)], self.calibration_y_vals)

    def set_min_max_voltage(self, voltage):
        if self.set_min_voltage(voltage) or self.set_max_voltage(voltage):
            self.set_slope_and_intercept_for_calibration()

    def set_max_voltage(self, voltage):
        if voltage > self.max_voltage and voltage != self.min_voltage:
            self.logger.debug("New Maximum Voltage Found")
            self.logger.debug("Old Maximum Voltage:" + str(self.max_voltage))
            self.logger.debug("Setting maximum_voltage to: " + str(voltage))
            self.max_voltage = voltage
            return True
        return False

    def set_min_voltage(self, voltage):
        if voltage < self.min_voltage:
            self.logger.debug("New Minimum Voltage Found")
            self.logger.debug("Old Minimum Voltage:" + str(self.min_voltage))
            self.logger.debug("Setting minimum_voltage to: " + str(voltage))
            self.min_voltage = voltage
            return True
        return False

    def set_min_max_voltage_from_previous_calibration(self):
        self.set_max_voltage_from_prior_calibration()
        self.set_min_voltage_from_prior_calibration()

    def set_max_voltage_from_prior_calibration(self):
        self.logger.info("Setting max_voltage based on previous slope and intercept.")
        self.max_voltage = -(self.m / self.b)

    def set_min_voltage_from_prior_calibration(self):
        self.logger.info("Setting min_voltage based on previous slope and intercept.")
        self.min_voltage = (self.m / (100 - self.b))

    def convert_voltage_to_output(self, voltage):
        return self.m * (1 / voltage + self.b)

    def convert(self, voltage):
        output = self.convert_voltage_to_output(voltage)
        self.set_min_max_voltage(voltage)
        return output



import json
from calibrators.soil_moisture_voltage_converter import SoilMoistureVoltageConverter
from calibrators import RawValueConverter


class RawValueDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, dct):
        type = dct['type']

        if type == SoilMoistureVoltageConverter.__name__:
            return SoilMoistureVoltageConverter(soil_moisture_sensor_serial_no=dct['soil_moisture_sensor_serial_no'],
                                                m=dct['m'], b=dct['b'], max_voltage=dct['max_voltage'],
                                                min_voltage=dct['min_voltage'])
        else:
            return RawValueConverter()



import json
from OpenHub.hardware_interfaces.hardware_interface import HardwareInterface
from OpenHub.hardware_interfaces.channels.json.channel_encoder import ChannelEncoder


class HardwareEncoder(json.JSONEncoder):
    channel_encoder = ChannelEncoder()

    def default(self, o):
        if isinstance(o, HardwareInterface):
            hardware_json = o.__dict__
            encoded_channels = []
            for channel in o.channels:
                encoded_channels.append(json.dumps(channel, cls=ChannelEncoder))

            hardware_json['channels'] = encoded_channels
            return hardware_json

        return json.JSONEncoder.default(self, o)



import argparse
import asyncio
import logging
import random
import string
import time

import aiohttp

from aiortc import RTCPeerConnection, RTCSessionDescription, VideoStreamTrack
from aiortc.contrib.media import MediaPlayer, MediaRecorder

pcs = set()


def transaction_id():
    return "".join(random.choice(string.ascii_letters) for x in range(12))


class JanusPlugin:
    def __init__(self, session, url):
        self._queue = asyncio.Queue()
        self._session = session
        self._url = url

    async def send(self, payload):
        message = {"janus": "message", "transaction": transaction_id(), "apisecret": "gardengnome"
                   }
        message.update(payload)
        async with self._session._http.post(self._url, json=message) as response:
            data = await response.json()
            assert data["janus"] == "ack"

        response = await self._queue.get()
        assert response["transaction"] == message["transaction"]
        return response


class JanusSession:
    def __init__(self, url):
        self._http = None
        self._poll_task = None
        self._plugins = {}
        self._root_url = url
        self._session_url = None

    async def attach(self, plugin_name: str) -> JanusPlugin:
        message = {
            "janus": "attach",
            "plugin": plugin_name,
            "transaction": transaction_id(),
            "apisecret": "gardengnome"
        }
        async with self._http.post(self._session_url, json=message) as response:
            data = await response.json()
            assert data["janus"] == "success"
            plugin_id = data["data"]["id"]
            plugin = JanusPlugin(self, self._session_url + "/" + str(plugin_id))
            self._plugins[plugin_id] = plugin
            return plugin

    async def create(self):
        self._http = aiohttp.ClientSession()
        message = {"janus": "create", "transaction": transaction_id(), "apisecret": "gardengnome"}
        async with self._http.post(self._root_url, json=message) as response:
            data = await response.json()
            assert data["janus"] == "success"
            session_id = data["data"]["id"]
            self._session_url = self._root_url + "/" + str(session_id)

        self._poll_task = asyncio.ensure_future(self._poll())

    async def destroy(self):
        if self._poll_task:
            self._poll_task.cancel()
            self._poll_task = None

        if self._session_url:
            message = {"janus": "destroy", "transaction": transaction_id(), "apisecret": "gardengnome"}
            async with self._http.post(self._session_url, json=message) as response:
                data = await response.json()
                assert data["janus"] == "success"
            self._session_url = None

        if self._http:
            await self._http.close()
            self._http = None

    async def _poll(self):
        while True:
            headers = {'Prefer': 'wait=30'}
            params = {"maxev": 1, 'apisecret': 'gardengnome'}
            async with self._http.get(self._session_url, headers=headers, params=params) as response:
                data = await response.json()
                if data["janus"] == "event":
                    plugin = self._plugins.get(data["sender"], None)
                    if plugin:
                        await plugin._queue.put(data)
                    else:
                        print(data)
            await asyncio.sleep(30)


recorder = MediaRecorder('/Users/ganonpierce/src/test/test0.mp4')


async def run(session):
    session = JanusSession("http://192.168.3.101:8088/janus")

    pc = RTCPeerConnection()

    await session.create()

    # join video room
    plugin = await session.attach("janus.plugin.streaming")
    response = await plugin.send(
        {
            "body": {"request": "watch",
                     "id": 1},
        }
    )
    print(response["plugindata"]["data"])
    # apply offer
    await pc.setRemoteDescription(
        RTCSessionDescription(
            sdp=response["jsep"]["sdp"], type=response["jsep"]["type"]
        )
    )

    # send answer
    await pc.setLocalDescription(await pc.createAnswer())

    print(str(pc.localDescription.sdp))
    response = await plugin.send(
        {
            "body": {"request": "start"},
            "jsep": {
                "sdp": pc.localDescription.sdp,
                "type": pc.localDescription.type
            },
        }
    )

    print(response)
    await recorder.start()


if __name__ == "__main__":
    # parser = argparse.ArgumentParser(description="Janus")
    # parser.add_argument("url", help="Janus root URL, e.g. http://localhost:8088/janus")
    # parser.add_argument(
    #     "--room",
    #     type=int,
    #     default=1234,
    #     help="The video room ID to join (default: 1234).",
    # ),
    # parser.add_argument("--play-from", help="Read the media from a file and sent it."),
    # parser.add_argument("--record-to", help="Write received media to a file."),
    # parser.add_argument("--verbose", "-v", action="count")
    # args = parser.parse_args()

    # if args.verbose:
    #     logging.basicConfig(level=logging.DEBUG)

    # create signaling and peer connection
    session = JanusSession("http://192.168.3.101:8088/janus")

    loop = asyncio.get_event_loop()
    try:
        loop.run_until_complete(
            run(session=session)
        )
    except KeyboardInterrupt:
        pass
    finally:

        if recorder is not None:
            loop.run_until_complete(recorder.stop())
        loop.run_until_complete(session.destroy())

        # close peer connections
        coros = [pc.close() for pc in pcs]
        loop.run_until_complete(asyncio.gather(*coros))

    while True:
        asyncio.sleep(1)

# DTLS HANDSHAKE DIFFERENT BETWEEN TWO PROBABLY ONLY HAPPENS WHEN YOU DO SOMETHING WITH THE TRACK (Or something that happens before you start using the track)


import uuid
import os
from hardware_interfaces.channels import ModProbeTemp
from hardware_interfaces.channels.veml7700_lux import VEML7700Lux
from hardware_interfaces.channels import VEML7700Light
from hardware_interfaces.channels.dht22_temp import DHT22Temp
from hardware_interfaces.channels import DHT22Humidity
from homekit_accessories import SoilMoistureSensor
from homekit_accessories.soil_temperature_sensor import SoilTemperatureSensor
from homekit_accessories import AirTemperatureSensor
from homekit_accessories.humidity import HumiditySensor
from homekit_accessories import LightSensor
from homekit_accessories.liquid_level_sensor import LiquidLevelSensor
from homekit_accessories.pressure_sensor import PressureSensor
from homekit_accessories import Pump
from homekit_accessories.etape import ETapeSensor

def first_run():
    initialize_homekit_interface_config()
    configure_hub_first_time()


def initialize_homekit_interface_config():
    homekit_interface_config_file = os.getenv('OPENHUB_HOMEKIT_CONFIG_FILE')
    if homekit_interface_config_file is None:
        from OpenHub import HOMEKIT_CONFIG_FILE

        homekit_interface_config_file = input(
            "What is the absolute path you would like to set for the homekit interface config?") or HOMEKIT_CONFIG_FILE
        if homekit_interface_config_file != HOMEKIT_CONFIG_FILE:
            "Please place this in your bash_profile: export OPENHUB_HOMEKIT_CONFIG_FILE=" + homekit_interface_config_file
            HOMEKIT_CONFIG_FILE = homekit_interface_config_file
            os.putenv('HOMEKIT_CONFIG_FILE', HOMEKIT_CONFIG_FILE)
    else:
        print(
            'Setting homekit config file to environment variable OPENHUB_HOMEKIT_CONFIG_FILE: ' + homekit_interface_config_file)


def configure_hub_first_time(sensor_hub):
    sensor_hub.add_name()
    sensor_hub.configure_soil_moisture_sensors()
    sensor_hub.configure_air_temp_humidity()
    sensor_hub.configure_soil_temp()
    sensor_hub.configure_light_level()
    sensor_hub.serial_no = str(uuid.uuid4())
    sensor_hub.config["serial_no"] = sensor_hub.serial_no
    sensor_hub.config["display_name"] = sensor_hub.display_name


def add_name(sensor_hub):
    if sensor_hub.display_name is None:
        sensor_hub.display_name = input("Please Add Display Name for OpenHub: ") or "OpenHub"


def interface_channel_with_homekit_accessory(sensor_hub, channel):
    if channel.__name__ == ModProbeTemp.__name__:
        accessory = setup_soil_temperature_sensor_first_time(channel)
    elif channel.__name__ == VEML7700Lux.__name__:
        accessory = setup_light_sensor_first_time(channel)
    elif channel.__name__ == VEML7700Light.__name__:
        accessory = setup_light_sensor_first_time(channel)
    elif channel.__name__ == DHT22Temp.__name__:
        accessory = setup_air_temp_sensor_first_time(channel)
    elif channel.__name__ == DHT22Humidity.__name__:
        accessory = setup_humidity_sensor_first_time(channel)
    else:
        accessory = None
        channel_type = str(input("What type of homekit accessory should use this channel??"))
        if channel_type == 'soilmoisture':
            accessory = setup_soil_moisture_sensor_first_time(channel)
        if channel_type == 'soiltemperature':
            accessory = setup_soil_temperature_sensor_first_time(channel)
        if channel_type == 'airtemperature':
            accessory = setup_air_temp_sensor_first_time(channel)
        if channel_type == 'humidity':
            accessory = setup_humidity_sensor_first_time(channel)
        if channel_type == 'light':
            accessory = setup_light_sensor_first_time(channel)
        if channel_type == 'liquidlevel':
            accessory = setup_liquid_level_sensor_first_time(channel)
        if channel_type == 'pressure':
            accessory = setup_pressure_sensor_first_time(channel)
        if channel_type == 'etape':
            accessory = setup_etape_sensor_first_time(channel)
        if channel_type == 'pump':
            accessory = setup_pump_first_time(channel)
    if accessory is not None:
        sensor_hub.add_accessory(accessory)


def setup_light_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return LightSensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_air_temp_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return AirTemperatureSensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_humidity_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return HumiditySensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_soil_temperature_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return SoilTemperatureSensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_soil_moisture_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return SoilMoistureSensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_etape_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return ETapeSensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_liquid_level_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return LiquidLevelSensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_pressure_sensor_first_time(channel):
    id = str(uuid.uuid4())
    return PressureSensor(serial_no=id, channel_interface_serial_no=channel.serial_no)


def setup_pump_first_time(channel):
    id = str(uuid.uuid4())
    return Pump(serial_no=id, channel_interface_serial_no=channel.serial_no)



import logging

from OpenHub import PiPicoGPI


class PiPicoGPO(PiPicoGPI):
    logger = logging.getLogger(__name__)

    def __init__(self,default = 'off', *args, **kwargs):
        self.default = default
        super(PiPicoGPO, self).__init__(*args, **kwargs)
